/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Filename: CV2TAB.C          Controls on regular PM Client Windows
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  This file contains a series of routines to manage dialog controls on a
  regular PM Client Window.  Along with a rexx command file which converts
  the output of the dialog editor (*.DLG) into a 'C' routine which transforms
  the dialogunits into window PELS, and the dialog control statements into
  WinCreateWindow calls.

  CV2TAB.C      This File,
                fnHandleWMChar()       Handles keyboard tabbing
                fnClearView()          Clears the client window of all controls
                fnGetWindowList()      Calculates a retcl around all the controls
                fnCenterView()         Resizes the FrameWindow to contain the Controls
                fnSetWindowPresFont()  Sets the font of any control
                fnSetWindowPresColor() Set the color of any window
                fnSetScrollType()      Sets either WinSetMultWindowPos or WinScrollWindow
                fnHandleHScroll()      Calcs the H Scroll increment, and scrolls the client
                fnHandleVScroll()      Calcs the V Scroll increment, and scrolls the client
                fnHandleWMSize()       Calcs the new scroll bar positions and scroll the clients
                fnScrollView()        (1) Calcs internally scroll increments and call swpscroll.
                fnSwpScroll()         (1) Base on scroll type, performs the scroll on the client
                fnDefaultPaint()       Paints a single background color for WM_PAINT
                 (1) == private functions

  CV2DTOW.CMD   DLG to WinCreateWindow Conversion command file

*/
/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */
/*  Include files                                                              */
/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */
#define INCL_BASE                       /* relevant parts of               */
#define INCL_DEV                        /* relevant parts of               */
#define INCL_GPI                        /* relevant parts of               */
#define INCL_WIN                        /* relevant parts of               */
#define INCL_PM                         /* relevant parts of               */
                                        /* the PM header file              */
#include <os2.h>                        /* PM header file                  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "cv2tab.h"

/* Private Functions */
MRESULT    fnScrollView(HWND hwnd, PSCROLLINFO pScl);
INT        fnSwpScroll(HWND hwnd, SWP aWindows[], LONG lX, LONG lY, INT iCount, BOOL bTYPE);

/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:     fnHandleWMChar()
Description:  Provides focus and nemonic processing for control windows that
              are children of a standard PM Client window; via the WM_CHAR
              system message.  Function is as described for
              the WinDefDialogProc() in the PM programming reference.
     Inputs:  WM_CHAR's message - hwnd, mp1, mp2
     Output:  none
    Returns:  Boolean TRUE if WM_CHAR was processed, else FALSE if not
    Effects:  Tabs entry fields, or click buttons controls, and sets the focus
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/
MRESULT      fnHandleWMChar(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
 ULONG  command = 0L;
 BOOL   bFound  = FALSE;
 ULONG  dlgcode = (ULONG)0;
 HWND   newhwnd = (HWND)0,
        fhwnd   = (HWND)0;
 HENUM  henum   = (HENUM)0;
 USHORT uChar   = (USHORT)0;
 HWND  hHscroll = (HWND)0;
 HWND  hVscroll = (HWND)0;

/**********************************************************************
 ** Start Mnemonic Management
 **  a. see if current control is a button, if yes send it a click and exit.
 **  b. check all controls for buttons with the default flag.
 **     - if found send a click, set the focus, and exit
 **********************************************************************
*/

 hVscroll = WinWindowFromID(WinQueryWindow(hwnd, QW_PARENT), FID_VERTSCROLL );
 hHscroll = WinWindowFromID(WinQueryWindow(hwnd, QW_PARENT), FID_HORZSCROLL );
 uChar   = SHORT1FROMMP(mp2);     /* Load the character */

 if( (SHORT1FROMMP(mp1) & KC_CHAR) && !(SHORT1FROMMP(mp1) & KC_VIRTUALKEY) &&
    !(SHORT1FROMMP(mp1) & KC_KEYUP) )
 {

  /* Test current control - matching mnemonic - Click if buttonclass   */
  fhwnd=WinQueryFocus(HWND_DESKTOP);
  if(WinSendMsg(fhwnd,WM_MATCHMNEMONIC,MPFROMSHORT(uChar),0L))
  {
   dlgcode = (ULONG)WinSendMsg(fhwnd, WM_QUERYDLGCODE,0L,0L);
   if (dlgcode & DLGC_BUTTON)
      WinSendMsg(fhwnd,BM_CLICK,0L,0L);

   return (MRESULT)TRUE;
  } /* end if button class   */

  /* Test all controls - matching mnemonic - SetFocus   */
  henum=WinBeginEnumWindows(hwnd);
  bFound = FALSE;                      /* exit this routine if found */
  newhwnd=WinGetNextWindow(henum);
  while( newhwnd )
  {
   if(WinSendMsg(newhwnd, WM_MATCHMNEMONIC, MPFROMSHORT(uChar), 0L))
   {
    WinSetFocus(HWND_DESKTOP, newhwnd);
    bFound = TRUE;                      /* match was found and acted upon */
    break;
   } // end if

   newhwnd=WinGetNextWindow(henum);
  } // end while
  WinEndEnumWindows(henum);

  if(bFound) return (MRESULT)TRUE;
  else return(WinDefWindowProc(hwnd, msg,mp1,mp2));

 } /* end if KC_CHAR for Mnemonic  */

/********************************************************************
 * Start Tab Management
 *  a. determine if the key is for the vertical scroll bar
 *  b. determine the group direction command
 *  c. if the key was enter or newline look for a default button and click it
 *  d. change the focus according to the group direction command
 ********************************************************************
*/
 if( !(SHORT1FROMMP(mp1) & KC_VIRTUALKEY) )   /* ignore none virtual keys */
   return(WinDefWindowProc(hwnd,msg,mp1,mp2));

 if(SHORT1FROMMP(mp1) & KC_KEYUP)      /* ignore key releases */
   return(WinDefWindowProc(hwnd,msg,mp1,mp2));

 /* Check for Shifed state for scroll bars */
 if ( SHORT1FROMMP(mp1) & KC_SHIFT )
 {
    if ( (SHORT2FROMMP(mp2) == VK_UP )      ||
         (SHORT2FROMMP(mp2) == VK_DOWN )    ||
         (SHORT2FROMMP(mp2) == VK_PAGEUP)   ||
         (SHORT2FROMMP(mp2) == VK_PAGEDOWN) )
    {
    if (hVscroll) return WinSendMsg (hVscroll, msg, mp1, mp2) ;
    }

    if ( (SHORT2FROMMP(mp2) == VK_LEFT) ||
         (SHORT2FROMMP(mp2) == VK_RIGHT ) )
    {
    if (hHscroll) return WinSendMsg (hHscroll, msg, mp1, mp2) ;
    }
 } /* end if shifted state */

 command=0L;
 switch(SHORT2FROMMP(mp2))
 {
  case VK_PAGEUP:
  case VK_PAGEDOWN:
       if (hVscroll) return WinSendMsg (hVscroll, msg, mp1, mp2) ;
       else return (MRESULT)FALSE;

  case VK_UP:
  case VK_LEFT:
       command = EDI_PREVGROUPITEM;
       break;

  case VK_RIGHT:
  case VK_DOWN:
       command = EDI_NEXTGROUPITEM;
       break;

  case VK_TAB:
       command = EDI_NEXTTABITEM;
       break;

  case VK_BACKTAB:
       command = EDI_PREVTABITEM;
       break;

  case VK_ENTER:
  case VK_NEWLINE:                         /* if enter - find the default button and click it. */
       newhwnd = WinQueryFocus(HWND_DESKTOP);
       dlgcode = (ULONG)WinSendMsg(newhwnd,WM_QUERYDLGCODE,0L,0L);
       if ((dlgcode & DLGC_PUSHBUTTON) || (dlgcode & DLGC_DEFAULT))
       {
          WinSendMsg(newhwnd,BM_CLICK,0L,0L);
       }
       else
       {
        henum=WinBeginEnumWindows(hwnd);
        newhwnd=WinGetNextWindow(henum);
        while( newhwnd )
        {
         dlgcode = (ULONG)WinSendMsg(newhwnd,WM_QUERYDLGCODE,0L,0L);
         if (dlgcode & DLGC_DEFAULT)
         {
          WinSendMsg(newhwnd,BM_CLICK,0L,0L);
          WinSetFocus ( HWND_DESKTOP,newhwnd);
          break;
         }
         newhwnd=WinGetNextWindow(henum);
        } // end while
        WinEndEnumWindows(henum);
        return (MRESULT)TRUE;
       } // end if else

 } /* end vk-keys switch  */

/********************************************************************
* now move the focus to the new control
*********************************************************************
*/
 if(command != 0L)
 {
   newhwnd = (HWND)WinEnumDlgItem(hwnd, WinQueryFocus(HWND_DESKTOP), command);
   WinSetFocus ( HWND_DESKTOP,newhwnd);

   dlgcode = (ULONG)WinSendMsg(newhwnd,WM_QUERYDLGCODE,0L,0L);
   if(dlgcode & DLGC_RADIOBUTTON) WinSendMsg(newhwnd,BM_CLICK,0L,0L);

   return (MRESULT)TRUE;
  } else return(WinDefWindowProc(hwnd,msg,mp1,mp2));

} /* end function fnHandleWMChar   */

/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Module Name:  ClearView()
Description:  Destroys all the controls that are children of the client.
     Inputs:  Clients handle
     Output:  none
    Returns:  actual number of windows cleared/destroyed
    Effects:  Every Window on the client is destroyed
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/
INT        fnClearView(HWND hwnd, PSCROLLINFO pScl)
{
 HWND  newhwnd    = (HWND)0;
 HENUM henum      = (HWND)0;
 INT   iCount     = (INT)0;


 if(pScl == (PSCROLLINFO)0)
    return  FALSE;

 henum=WinBeginEnumWindows(hwnd);

 newhwnd=WinGetNextWindow(henum);
 while( newhwnd )
  {
    WinDestroyWindow(newhwnd);
    iCount++;
    newhwnd=WinGetNextWindow(henum);
  }

 WinEndEnumWindows(henum);

 pScl->iWindowCount == 0;

 return iCount;
} /* end clearview */

/* Ŀ */
/*  Set Window Colors using Presentation Parms                                */
/*  */
BOOL     fnSetWindowPresColor(HWND hwndTarget,ULONG ulWhat, ULONG ulCLR)
{
 ULONG  ulColor = ulCLR;
 LONG   lrc;

  /* ulWhat types - PP_FOREGROUNDCOLOR                         */
  /* ulCLR  values  CLR_RED  CLR_BLUE etc.                     */


 if((ulCLR == 0L) || (ulWhat == 0L))
   return FALSE;

 lrc = (LONG)WinSetPresParam (hwndTarget, ulWhat,(ULONG)sizeof(ULONG), &ulColor);

 return (BOOL)lrc;
}


/* Ŀ*/
/*  Set Window Font using Presentation Parms                                 */
/* */
BOOL    fnSetWindowPresFont(HWND hwndTarget,ULONG ulWhat, PSZ pszFontName)
{
 BOOL brc = FALSE;
  /* ulWhat types - PP_FONTNAMESIZE                            */
  /* pszFontName format - "10.Helv"  "12.Tms Rmn" "14.Tms Rmn" */

 if((pszFontName == (PSZ)NULL) || (ulWhat == 0L))
   return FALSE;

 brc = WinSetPresParam (hwndTarget, ulWhat,
           (ULONG)((ULONG)strlen(pszFontName)+(ULONG)1), (PSZ)pszFontName);
 return brc;
}

/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Module Name:  GetWindowList()
Description:  Initializes the PSCROLLINFO structure and generates the view/panel
              initial position on the client window.  Used immediately after
              creating a new view/panel on the client area.

              Primary role - Determines the rectangle area of all the controls
              on the client area as if the where a single object.  Makes it
              easier to compare this logical rectangle to the rectangle of the
              client area, when calculating the scrolling amounts.

     Inputs:  Clients handle
     Output:  Visual change in client area - panel positions to top left corner
    Returns:  interger count of the number of controls in current view/panel.
    Effects:  PSCROLLINFO structure.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/
INT        fnGetWindowList(HWND hwnd,  PSCROLLINFO pScl)
{
 PSWP  aWindows;
 SWP   swp;
 HWND  newhwnd  = (HWND)0;
 HENUM henum    = (HWND)0;
 INT   iCount   = 0;
 LONG  x        = (LONG)0,
       y        = (LONG)0,
       cx       = (LONG)0,
       cy       = (LONG)0;

 if(pScl == (PSCROLLINFO)0)
    return  FALSE;

 aWindows = pScl->aControls;

 henum=WinBeginEnumWindows(hwnd);

 newhwnd=WinGetNextWindow(henum);
 while( newhwnd )
 {
    if(WinQueryWindowPos(newhwnd, (PSWP)&((aWindows)[iCount]) ))
      {
        x = min( x, (aWindows)[iCount].x);
        y = min( y, (aWindows)[iCount].y);
       cx = max(cx, ((aWindows)[iCount].x + (aWindows)[iCount].cx));
       cy = max(cy, ((aWindows)[iCount].y + (aWindows)[iCount].cy));

       (aWindows)[iCount].fl = SWP_MOVE | SWP_NOADJUST | SWP_NOREDRAW;
       iCount++;
       if( iCount == MAX_CONTROLS)            // protect aSWP[] storage
         break;
      }
  newhwnd=WinGetNextWindow(henum);
  } /* end while */
 WinEndEnumWindows(henum);

 pScl->iWindowCount = iCount;
// **********

 cx -= x;      // True cx
 cy -= y;

 // Pad Top & Right edges
 cx += WinQuerySysValue (HWND_DESKTOP, SV_CXSIZEBORDER);
 cy += WinQuerySysValue (HWND_DESKTOP, SV_CYSIZEBORDER);

 pScl->cx = cx;
 pScl->cy = cy;
 pScl->x = x;
 pScl->y = y;
 pScl->xRange = 0;
 pScl->yRange = 0;

 WinQueryWindowPos(hwnd, (PSWP)&swp);            /* Home the view */
 pScl->XPos = cx - swp.cx ;
 pScl->YPos = swp.cy - cy ;

 fnScrollView(hwnd, pScl);
 return iCount;
}

/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Module Name:  CenterView()
Description:  Positions the Frame in the center of the screen and sized to view.

     Inputs:  Clients handle and PSCROLLINFO structure
     Output:  none
    Returns:  BOOL false - n/a
    Effects:  move, size, and make visible the frame window
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/
BOOL       fnCenterView(HWND hwnd, PSCROLLINFO pScl )
{
 LONG lx     = 0L,
      ly     = 0L,
      lcx    = 0L,
      lcy    = 0L,
      lwidth = 0L,
      ldepth = 0L;
 BOOL brc    = FALSE;
 HWND  hHscroll = (HWND)0;
 HWND  hVscroll = (HWND)0;

 if(pScl == (PSCROLLINFO)0)
    return  FALSE;

 hVscroll = WinWindowFromID(WinQueryWindow(hwnd, QW_PARENT), FID_VERTSCROLL );
 hHscroll = WinWindowFromID(WinQueryWindow(hwnd, QW_PARENT), FID_HORZSCROLL );

 /* Calculate FRAME new size   ******************************************
 */
 /* Calc cx of client and frame attributes */
 lcx = pScl->cx + (2 * WinQuerySysValue (HWND_DESKTOP, SV_CYSIZEBORDER));
 if ( hVscroll ) lcx += WinQuerySysValue (HWND_DESKTOP, SV_CXVSCROLL) ;

 /* Calc cy of client and frame attributes */
 lcy  = pScl->cy + (2 * WinQuerySysValue (HWND_DESKTOP, SV_CYSIZEBORDER));
 lcy += WinQuerySysValue (HWND_DESKTOP, SV_CYTITLEBAR);
 lcy += WinQuerySysValue (HWND_DESKTOP, SV_CYMENU);
 if ( hHscroll ) lcy += WinQuerySysValue (HWND_DESKTOP, SV_CYHSCROLL);

 /* Query width and depth of Desktop ************************************
 */
 lwidth = WinQuerySysValue( HWND_DESKTOP, SV_CXSCREEN );
 ldepth = WinQuerySysValue( HWND_DESKTOP, SV_CYSCREEN );

 /* Calculate center position  for frame                                  */
  lwidth -= lcx;
  ldepth -= lcy;
  lx  = (lwidth / 2) ;
  ly  = (ldepth / 2) ;

  brc = (ULONG)WinSetWindowPos( WinQueryWindow(hwnd, QW_PARENT), HWND_TOP,
                               lx, ly, lcx, lcy,
                               SWP_MOVE | SWP_SHOW | SWP_ACTIVATE | SWP_SIZE);
  return brc;
}

/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Module Name:  SetScrollType()
Description:  Changes the scrolling method used between pel mode
              using WinScrollWindow, and block mode using WinSetMultiWindowPos.
     Inputs:  Client Handle
           :  [BITSCROLL | SWPSCROLL] constant.  defined as TRUE/FALSE
     Output:  none
    Returns:  Boolean TRUE if mode was changed, else FALSE if access was denied
    Effects:  The way the scrollbar action respond to mouse requests.
           :  Modifies PSCROLLINFO struc in clients windowword alloc_scroll
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/
BOOL       fnSetScrollType(HWND hwnd, BOOL bTYPE, PSCROLLINFO pScl)
{

 if(pScl == NULL)
    return FALSE;

 pScl->bTYPE = bTYPE;
 return TRUE;
}


/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Module Name:  HandleVScroll()
Description:  Determines the scroll increment required for the current
              keyboard/mouse action.

     Inputs:  Clients handle and mp2 from WM_?SCROLL message.
     Output:  none
    Returns:  mresult false - n/a
    Effects:  move the view/panel on the client area
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/
MRESULT    fnHandleVScroll(HWND hwnd, MPARAM mp2, PSCROLLINFO pScl)
{

 if(pScl == NULL)
   return (MRESULT)FALSE;

 if(pScl->iWindowCount == 0)
   return (MRESULT)FALSE;


 switch (SHORT2FROMMP (mp2))
 {
  case  SB_LINEUP :
     pScl->YPos -= WNDROW;
     break;
  case  SB_LINEDOWN :
     pScl->YPos += WNDROW;
     break;
  case  SB_PAGEUP :
     pScl->YPos -= pScl->cy-2 * WNDROW;
     break;
  case  SB_PAGEDOWN :
     pScl->YPos += pScl->cy-2 * WNDROW;
     break;
  case  SB_SLIDERPOSITION :
     pScl->YPos = (LONG)(SHORT1FROMMP(mp2)) - pScl->yRange;
     break;
  case  SB_SLIDERTRACK:
     if(pScl->bTYPE == BITSCROLL)         // only if winscrollwindow
      pScl->YPos = (LONG)(SHORT1FROMMP(mp2)) - pScl->yRange;
     break;
  default  :
     return (MRESULT)FALSE;
 } // end switch

 fnScrollView(hwnd, pScl);

 return (MRESULT)FALSE;
}

/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Module Name:  HandleHScroll()
Description:  Determines the scroll increment required for the current
              keyboard/mouse action.

     Inputs:  Clients handle and mp2 from WM_?SCROLL message.
     Output:  none
    Returns:  mresult false - n/a
    Effects:  move the view/panel on the client area
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/
MRESULT    fnHandleHScroll(HWND hwnd, MPARAM mp2, PSCROLLINFO pScl)
{

 if(pScl == NULL)
   return (MRESULT)FALSE;

 if(pScl->iWindowCount == 0)
   return (MRESULT)FALSE;

 switch (SHORT2FROMMP (mp2))
 {
  case  SB_LINELEFT :
     pScl->XPos += WNDCOL;
     break;
  case  SB_LINERIGHT :
     pScl->XPos -= WNDCOL;
     break;
  case  SB_PAGELEFT :
     pScl->XPos += pScl->cx-2*WNDCOL;
     break;
  case  SB_PAGERIGHT :
     pScl->XPos -= pScl->cx-2*WNDCOL;
     break;
  case  SB_SLIDERPOSITION :
     pScl->XPos = -(LONG)(SHORT1FROMMP(mp2));
     break;
  case  SB_SLIDERTRACK:
     if(pScl->bTYPE == BITSCROLL)         // only if winscrollwindow
        pScl->XPos = -(LONG)(SHORT1FROMMP(mp2));
     break;
  default  :
     return (MRESULT)FALSE;
 } // end switch

 fnScrollView(hwnd, pScl);

 return (MRESULT)FALSE;
} // end function

/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Module Name:  HandleWMSize()
Description:  Forces the scrolling system to honor the top left corner as
              the home position anchor.  Before this; adjusting the bottom
              of the client windows changed its 0,0 reference for the control
              windows - causing all the controls to move with the clients 0,0.

     Inputs:  Clients handle and mp1/mp2 from WM_SIZE message.
     Output:  none
    Returns:  mresult false - n/a
    Effects:  move the view/panel on the client area
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/
MRESULT    fnHandleWMSize(HWND hwnd, MPARAM mp1, MPARAM mp2, PSCROLLINFO pScl)
{
  // mp1=old, mp2=new
  // Adjusting Ypos fakes out a vertical scroll
  // we don't care about the horiz adjustment

 if(pScl == NULL)
   return  FALSE;

 pScl->YPos += (LONG)(SHORT2FROMMP(mp2) - SHORT2FROMMP(mp1));

 fnScrollView(hwnd, pScl);

 return (MRESULT)FALSE;
}

/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Module Name:  ScrollView()
Description:  Positions the view/panel relative to the lower-left corner
              of the Client Window - also manages scroll bar(s) if required.

     Inputs:  Clients handle and PSCROLLINFO structure
           :  the other routines have prepared the scrollinfo structure for this
           :  routine to do the actual scrolling.
     Output:  none
    Returns:  mresult false - n/a
    Effects:  move the view/panel on the client area
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/
MRESULT    fnScrollView(HWND hwnd, PSCROLLINFO pScl)
{
 HWND         hHscroll = (HWND)0;
 HWND         hVscroll = (HWND)0;
 SWP          swp;

 // Query Client window position
 WinQueryWindowPos(hwnd, (PSWP)&swp);
 hVscroll = WinWindowFromID(WinQueryWindow(hwnd, QW_PARENT), FID_VERTSCROLL );
 hHscroll = WinWindowFromID(WinQueryWindow(hwnd, QW_PARENT), FID_HORZSCROLL );

// **********************************************************
// if panel cy is smaller than client cy turn off vert scroll
// **********************************************************
 if (pScl->cy <= swp.cy)
 {
  pScl->YPos = swp.cy-(*pScl).cy; // Increment = Client.cy minus Panel.cy
  (*pScl).yRange = 0L;             // Scrolling off if no range

      // Disabled
      // Cua Modification 6/15/91  - Force Large Marker
  if(hVscroll != (HWND)0)
  {
   WinSendMsg( hVscroll,
               SBM_SETSCROLLBAR,
               MPFROMSHORT(10), MPFROM2SHORT(1, 10));
   WinSendMsg( hVscroll,
               SBM_SETTHUMBSIZE,
               MPFROM2SHORT(10, 10), (MPARAM)0);
  }
 } else {

         // enabled scroll bar
         /**********************************************************************/
         /* adjust Y if nessecary to insure proper scroll range                */
         /**********************************************************************/
         if (pScl->YPos > 0L)
         {
            pScl->YPos = 0L;
         }else{
               if (pScl->YPos < (swp.cy-(*pScl).cy))
               {
                  pScl->YPos = swp.cy-(*pScl).cy;
               }
              }

              (*pScl).yRange = (*pScl).cy-swp.cy;

         /**********************************************************************/
         /* adjust the position & range of the scroll bar                      */
         /**********************************************************************/

         if(hVscroll != (HWND)0)
         {
            WinSendMsg( hVscroll,
                        SBM_SETSCROLLBAR,
                        (MPARAM)((*pScl).yRange + pScl->YPos),
                        MPFROM2SHORT(0, (SHORT)(*pScl).yRange));

            WinSendMsg( hVscroll,
                        SBM_SETTHUMBSIZE,
                        MPFROM2SHORT((SHORT)swp.cy, (SHORT)(*pScl).cy),
                        (MPARAM)0);
         }
          // ********************
        } // end( vertical )if
          // ********************

// ***********************************************
// If client wider than panel turn off horz scroll
// ***********************************************
   if ((*pScl).cx <= swp.cx)
    {
      pScl->XPos = 0L;

      // Disable the horizontal scroll bar
      // Cua Modification 6/15/91
      if(hHscroll != (HWND)0)
        {
         WinSendMsg( hHscroll,
                    SBM_SETSCROLLBAR,
                    MPFROMSHORT(10), MPFROM2SHORT(1, 10));

         WinSendMsg( hHscroll,
                    SBM_SETTHUMBSIZE,
                    MPFROM2SHORT(10, 10), (MPARAM)0);
        }
   } else {

      // enable scroll bar
      /**********************************************************************/
      /* adjust x if necessary to insure proper scroll range                */
      /**********************************************************************/
      if (pScl->XPos > 0L)
       {
         pScl->XPos = 0L;
       }else{
         if (pScl->XPos < (swp.cx-(*pScl).cx))
            pScl->XPos = swp.cx-(*pScl).cx;
       }

      (*pScl).xRange = (*pScl).cx-swp.cx;

      /**********************************************************************/
      /* adjust the position & range of the scroll bar                      */
      /**********************************************************************/
      if(hHscroll != (HWND)0)
        {
         WinSendMsg(hHscroll,
                    SBM_SETSCROLLBAR,
                    (MPARAM)(abs(pScl->XPos)),
                    MPFROM2SHORT(0, (SHORT)(*pScl).xRange));

         WinSendMsg(hHscroll,
                    SBM_SETTHUMBSIZE,
                    MPFROM2SHORT((SHORT)swp.cx, (SHORT)(*pScl).cx),
                    (MPARAM)0);
        }
     // **************
   } // end( horz )if
     // **************

// ***************************************************
// If View is not at the calc'ed position lets move it
// ***************************************************
   if ((*pScl).x != pScl->XPos || (*pScl).y != pScl->YPos)
    {
      fnSwpScroll(hwnd, pScl->aControls, pScl->XPos - (*pScl).x,
                                       pScl->YPos - (*pScl).y,
                                       pScl->iWindowCount, pScl->bTYPE);

      (*pScl).x = pScl->XPos;
      (*pScl).y = pScl->YPos;
    }

   pScl->XPos = pScl->x;
   pScl->YPos = pScl->y;

   return (MRESULT)FALSE;
}
/* End PRIVATE FUNCTION : ScrollView                                          */

/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Module Name:  SwpScroll()
Description:  Internal worker routine.
              Moves the logical view/panel on the client area.

     Inputs:  Client handle,
           :  SWP list of controlwindows,
           :  X and Y scroll increments/offsets
           :  Scrolling method flag
     Output:  Adjusts the SWP window list to track scroll their new position.
    Returns:  api return code of winscrollwindow/winsetmultiwindowpos.
    Effects:  move either the client area origin or moves the SWP window list.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/
INT        fnSwpScroll(HWND hwnd, SWP aWindows[], LONG lX, LONG lY, INT iCount, BOOL bTYPE)
{
 LONG  lrc = 0;
 register x = 0;

 if( iCount >= MAX_CONTROLS)            // protect aSWP[] storage
         iCount = MAX_CONTROLS - 1;

 for ( x = 0; x < iCount; x++)
      {
       (aWindows)[x].x += lX;           // track the movements in SWP
       (aWindows)[x].y += lY;
      }

 if(bTYPE == SWPSCROLL)
   {
     lrc = (LONG)WinSetMultWindowPos (WinQueryAnchorBlock(hwnd),
                                      aWindows, (ULONG)iCount);
     WinInvalidateRect(hwnd, NULL, TRUE);
   } else {
            lrc =  WinScrollWindow(hwnd, lX, lY,
                                     (PRECTL)0, (PRECTL)0,
                                     (HRGN)0,   (PRECTL)0,
                                     SW_INVALIDATERGN | SW_SCROLLCHILDREN);
          }
 return lrc;
}
/******************************************************************************
  ProcessDefaultPaint - Default WM_Paint Processing
  -- return TRUE if OK.
******************************************************************************/
MRESULT    fnDefaultPaint(HWND hwnd, LONG SysColor)
{
 HPS    hps   = (HPS)0;
 BOOL   bRc   = FALSE;
 RECTL  rectl;               /* Window rectangle   */

 hps = WinBeginPaint (hwnd, (HPS)0, &rectl);
           WinFillRect (hps, (PRECTL)&rectl, SysColor);
 bRc = WinEndPaint (hps);

 return (MRESULT)bRc;
}

